/* This file is part of VisualDFT.
 *
 * VisualDFT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * VisualDFT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VisualDFT; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package ch.educeth.visualdft;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.geom.*;
import java.awt.image.BufferedImage;
import java.awt.geom.AffineTransform;
import javax.swing.border.*;

/*
 * Dies ist die Frequency-Time-View abgeleitet von Jpanel. Sie ist verantwortlich fr: 
 * 1. das layout des frequenz-zeit-fensters
 * 2. visualisierung des "spektrums" eines signals. Entweder eingegeben vom benutzer,
 *    oder von der fourier transformation selbst berechnet.
 * 3. alle listeners die dieses fenster betreffen. 
 * 4. notifikation anderer komponenten bei vernderungen der felder durch den benutzer 
 */

public class AFView extends JPanel{

	private static final int WIDTH=395,HEIGHT=280;
	private int pressOut;
	private DftRoot dftRoot;

	final private int RANDX=40,RANDY=40;
	private AffineTransform at=new AffineTransform() ;
	private AffineTransform atInv;
	private AffineTransform turnedTextTransformation=new AffineTransform();
	private Point2D point= new Point2D.Double(0.0,0.0);

	private Border outerBorder= BorderFactory.createLineBorder(Color.black);
	private TitledBorder titleBorder = BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), 
		" D I S C R E T E   F O U R I E R   T R A N S F O R M ");

	final private Color backColor = new Color(0.75f, 0.80f, 0.9f);

	final private Font thisFont = new Font("Arial", Font.PLAIN, 12);


	private Rect [] rectArray;
	private Rectangle2D[] barArray;

	private Rectangle chartRect=new Rectangle(RANDX,RANDY,WIDTH-2*RANDX,HEIGHT-2*RANDY);
	private MyChart chart=new MyChart(chartRect,11,1,1,0,1,1,0,0);       
   

	MouseAdapter mouseAdapter=new MouseAdapter() {
   
			public void mousePressed(MouseEvent e) {
				int ii;

				point.setLocation((double)e.getX(), (double)e.getY()) ;								point=atInv.transform(point,point);				

				for (ii=0 ; ii<(dftRoot.ANZ_SAMPLES/2+1) ; ii++) {

					if(rectArray[ii].contains(point.getX(), point.getY())) {
					// Rect getroffen
					pressOut = ii+1;
					}
				}

			}
     			public void mouseReleased(MouseEvent e) { pressOut = 0; }
	};



	MouseMotionListener mouseMotionListener= new MouseMotionListener() {

			public void mouseMoved(MouseEvent e) {}

    			public void mouseDragged(MouseEvent e) {
			double le=0,cosKoeff=0,sinKoeff=0,ytransf=0;


			if(pressOut>0){
			    double pY=0.0;
			    point.setLocation((double)e.getX(), (double)e.getY()) ;						    	    point=atInv.transform(point,point);
			
			    ytransf=point.getY();

			    if (ytransf>0.975) {ytransf=0.975;}
			    if (ytransf<1.0E-8) {ytransf=1.0E-8;}

			    rectArray[pressOut-1].setYKoord(ytransf);

			cosKoeff=dftRoot.getAmplFreq(0,pressOut-1); 
			sinKoeff=dftRoot.getAmplFreq(1,pressOut-1);
			le=Math.sqrt(cosKoeff*cosKoeff+sinKoeff*sinKoeff);

			
			if (le != 0){
				dftRoot.setAmplFreqTupple(pressOut-1,cosKoeff/le*ytransf,sinKoeff/le*ytransf);
			} else {
				dftRoot.setAmplFreqTupple(pressOut-1,ytransf,0.0);
			}

			dftRoot.dftInverse();
			dftRoot.redrawall();
			
			} else {  
	
			} 
   		    }
	};




	private void initializeTransformations(){
		double scaleFaktorX=0.0;
		double scaleFaktorY=0.0;

		at.translate((double) RANDX,((double)(HEIGHT-RANDY)));

		scaleFaktorX=(WIDTH-2*((double) RANDX))/(dftRoot.ANZ_SAMPLES/2);
		scaleFaktorY=-(HEIGHT-2*((double) RANDY))/1.0;

		at.scale(scaleFaktorX,scaleFaktorY);
		try{
		atInv=at.createInverse(); 
                } catch (Exception e) {
   		System.out.println("No valid inverse Transformation");}
		
		turnedTextTransformation.rotate(Math.toRadians(-90));
	}


	public AFView(DftRoot dftRootA){
		this.dftRoot=dftRootA;
		rectArray = new Rect [dftRoot.ANZ_SAMPLES/2+1];
		barArray = new Rectangle2D.Double [dftRoot.ANZ_SAMPLES/2+1];

		titleBorder.setTitleJustification(TitledBorder.LEFT);

		initializeTransformations();

		int i;
		for (i=0 ; i<(dftRoot.ANZ_SAMPLES/2+1) ; i++) {
		rectArray[i]= new Rect(0.02d); 
		rectArray[i].setXKoord((double) i);
		} 
		
		for (i=0 ; i<(dftRoot.ANZ_SAMPLES/2+1) ; i++) {
		barArray[i]= new Rectangle2D.Double(0,0,0,0);
		} 

		addMouseListener(mouseAdapter);
 	    	addMouseMotionListener(mouseMotionListener);

	}

	
	private void paintdots(Graphics2D g2){
		int i;
		for (i=0 ; i<(dftRoot.ANZ_SAMPLES/2+1) ; i++) {
	   	g2.fill(rectArray[i]);
		}
	}	


	private void paintBars(Graphics2D g2){
		int i;

		g2.setColor(Color.blue);
	    	g2.setStroke(new BasicStroke(0.01f));

		for (i=0 ; i<(dftRoot.ANZ_SAMPLES/2+1) ; i++) {
			barArray[i].setRect(i-0.05, 0, 0.1, rectArray[i].getYKoord() );
			g2.fill( barArray[i] );
		}

		g2.setColor(Color.orange);

		
		if ((dftRoot.Upper>-1) & (dftRoot.Under>0)) {
		for (i=dftRoot.Under-1; i< dftRoot.Upper ; i++) {				
			barArray[i].setRect(i-0.05, 0, 0.1, rectArray[i].getYKoord() );
			g2.fill( barArray[i] );
		}
		}

	}




	private void updateDots(){
		int i;
		double cosKoeff,sinKoeff;

		for (i=0 ; i<(dftRoot.ANZ_SAMPLES/2+1) ; i++) {

			cosKoeff=dftRoot.getAmplFreq(0,i);
			sinKoeff=dftRoot.getAmplFreq(1,i);

			rectArray[i].setYKoord(Math.sqrt(cosKoeff*cosKoeff+sinKoeff*sinKoeff));
		}

	}

	private void drawBorders(Graphics2D g2){
		AffineTransform saveXform = g2.getTransform(); 

		outerBorder.paintBorder(this, g2, 0, 0, WIDTH, HEIGHT);
	    	titleBorder.paintBorder(this, g2, 0, 0, WIDTH, HEIGHT);
          
	    	g2.transform(turnedTextTransformation);

		g2.setFont(thisFont);
	    	g2.drawString("Amplitude", -90, 13); 
          
	    	g2.setTransform(saveXform);

		g2.drawString("Frequency", WIDTH-90, HEIGHT-12); 
		
		g2.clipRect(RANDX-10,RANDY,WIDTH-2*RANDX+20,HEIGHT-2*RANDY+10);
	}



        
	public void paintComponent(Graphics g){
	   // super.paintComponent(g);
	    Graphics2D g2 = (Graphics2D)g;
	    
	    setSize(WIDTH,HEIGHT);

	    AffineTransform saveXform = g2.getTransform();

            // Clears the shape that was previously drawn.
            g2.setColor(backColor);
  	    g2.fillRect(0, 0, WIDTH, HEIGHT);
            g2.setColor(Color.white);
  	    g2.fill(chartRect);


	    chart.drawGrid(g2);

	    drawBorders(g2);

	    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

	    g2.transform(at);

	    updateDots();

	    paintBars(g2);
	
	    g2.setColor(Color.darkGray);
	    g2.setStroke(new BasicStroke(2.0f));

	    paintdots(g2);

	    g2.setTransform(saveXform);
         
	}
}