/* This file is part of VisualDFT.
 *
 * VisualDFT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * VisualDFT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VisualDFT; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package ch.educeth.visualdft;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.geom.*;
import java.awt.image.BufferedImage;
import javax.swing.border.*;

/*
 * Dies ist die Amplitude-Time-View abgeleitet von Jpanel. Sie ist verantwortlich fr: 
 * 1. das layout des amplituden-zeit-fensters
 * 2. visualisierung des "signals" . Entweder eingegeben vom benutzer,
 *    oder von der (inversen) fourier transformation selbst berechnet.
 * 3. alle listeners die dieses fenster betreffen. 
 * 4. notifikation anderer komponenten bei vernderungen der felder durch den benutzer 
 */


public class ATView extends JPanel{
	 
	private static final int WIDTH=395,HEIGHT=280;
        private static final int PREC=441;
	private int pressOut;

	private static final int RANDX=40,RANDY=40;
	private AffineTransform at=new AffineTransform() ;
	private AffineTransform atInv;
	private AffineTransform turnedTextTransformation=new AffineTransform();
	private Point2D point= new Point2D.Double(0.0,0.0);

	private static final Border outerBorder= BorderFactory.createLineBorder(Color.black);
	private static final TitledBorder titleBorder=BorderFactory.createTitledBorder(
						BorderFactory.createEtchedBorder(), " S I G N A L ");
	
	private static final Color BACKCOLOR = new Color(0.75f, 0.85f, 0.95f);
	private static final Font STDFONT = new Font("Arial", Font.PLAIN, 12);

	private Dot [] dotArray;

	private Rectangle chartRect=new Rectangle(RANDX,RANDY,WIDTH-2*RANDX,HEIGHT-2*RANDY);
	private MyChart chart=new MyChart(chartRect,1,0,1,-1,1,1,0,0);

	private GeneralPath graphShape= new GeneralPath();
	private GeneralPath graphShape2= new GeneralPath();
	private DftRoot dftRoot;


	MouseAdapter mouseAdapter=new MouseAdapter() {
   

			public void mousePressed(MouseEvent e) {
				int ii;

				point.setLocation((double)e.getX(), (double)e.getY()) ;									point=atInv.transform(point,point);

				for (ii=0 ; ii<(dftRoot.ANZ_SAMPLES) ; ii++) {

					if( dotArray[ii].contains(point.getX(), point.getY()) ) {
					// Dot getroffen!
					pressOut = ii+1;
					}
				}

			}
     			public void mouseReleased(MouseEvent e) { pressOut = 0; }
			
	};

	MouseMotionListener mouseMotionListener= new MouseMotionListener() {

			public void mouseMoved(MouseEvent e) {}

    			public void mouseDragged(MouseEvent e) {
			if(pressOut>0){
				double pY=0.0;
				point.setLocation((double)e.getX(), (double)e.getY()) ;									point=atInv.transform(point,point);				
				pY=point.getY();
				if (pY>0.95) {pY=0.95;}
				if (pY<-0.95) {pY=-0.95;}


				dotArray[pressOut-1].setYKoord(pY);

				dftRoot.setAmplTimePoint(pressOut-1,pY);

				dftRoot.dftForward();
				dftRoot.redrawall();
			} else {  
	
			} 
   		    }
	};


	private void initializeTransformations(){
		double scaleFaktorX=0.0;
		double scaleFaktorY=0.0;

		at.translate((double) RANDX,((double)HEIGHT)/2);

		scaleFaktorX=(WIDTH-2*((double) RANDX))/(2*Math.PI*(1.0-1.0/(double)dftRoot.ANZ_SAMPLES));
		scaleFaktorY=-(HEIGHT-2*((double) RANDY))/2.0;

		at.scale(scaleFaktorX,scaleFaktorY);
		try{
		atInv=at.createInverse(); 
                } catch (Exception e) {
   		System.out.println("No inverse transformation");}

		turnedTextTransformation.rotate(Math.toRadians(-90));
		
	}



	public ATView(DftRoot dftRootA){
		int i;
		dftRoot=dftRootA;

		dotArray=new Dot [dftRoot.ANZ_SAMPLES];
		
		titleBorder.setTitleJustification(TitledBorder.LEFT);

		initializeTransformations();
		
		for (i=0 ; i<(dftRoot.ANZ_SAMPLES) ; i++) {
			dotArray[i]= new Dot(0.04);
			dotArray[i].setXKoord(i*dftRoot.DELTA);
		} 
	
		addMouseListener(mouseAdapter);
  	    	addMouseMotionListener(mouseMotionListener);

	}



	private void paintdots(Graphics2D g2){
		int i;
		for (i=0 ; i<(dftRoot.ANZ_SAMPLES) ; i++) {
		g2.fill(dotArray[i]);
		}
	}	

	private void updateDots(){
		int i;
		for (i=0 ; i<(dftRoot.ANZ_SAMPLES) ; i++) {
			dotArray[i].setYKoord(dftRoot.getAmplTimePoint(i));
		}
	}

  
	private void drawBorders(Graphics2D g2){
		AffineTransform saveXform = g2.getTransform(); 

		outerBorder.paintBorder(this, g2, 0, 0, WIDTH, HEIGHT);
	    	titleBorder.paintBorder(this, g2, 0, 0, WIDTH, HEIGHT);
          
	    	g2.transform(turnedTextTransformation);

		g2.setFont(STDFONT);
	    	g2.drawString("Amplitude", -90, 13); 
          
	    	g2.setTransform(saveXform);

		g2.drawString("Time", WIDTH-60, HEIGHT-12); 
		
		g2.clipRect(RANDX-10,RANDY,WIDTH-2*RANDX+20,HEIGHT-2*RANDY);
	}


	public void paintComponent(Graphics g){
	    // super.paintComponent(g);
	    Graphics2D g2 = (Graphics2D)g;
	    
	    setSize(WIDTH,HEIGHT);
	
	    AffineTransform saveXform = g2.getTransform();    
	 
            // Clears the shape that was previously drawn.

	    g2.setColor(BACKCOLOR);
  	    g2.fillRect(0, 0, WIDTH, HEIGHT);
            g2.setColor(Color.white);
  	    g2.fill(chartRect);


	    chart.drawGrid(g2);

	    drawBorders(g2);
          

	    dftRoot.CalcGraph(PREC,graphShape2,dftRoot.Under-1,dftRoot.Upper-1);
	    graphShape2.transform(at);

	    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
	    g2.setColor(Color.orange);
	    g2.setStroke(new BasicStroke(1.25f));
	    g2.draw(graphShape2);


	    dftRoot.CalcGraph(PREC,graphShape,0,dftRoot.ANZ_SAMPLES/2);

	    graphShape.transform(at);

	    g2.setColor(Color.blue);
	    g2.setStroke(new BasicStroke(1.5f));
	    g2.draw(graphShape);

	    g2.transform(at);

	    g2.setColor(Color.black);
	    g2.setStroke(new BasicStroke(2.0f));

	    updateDots(); 
	    paintdots(g2);

            g2.setTransform(saveXform);     


	}
}