/* This file is part of VisualDFT.
 *
 * VisualDFT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * VisualDFT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VisualDFT; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package ch.educeth.visualdft;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.AbstractButton;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.ImageIcon;
import javax.swing.event.*;
import javax.swing.*;
import javax.swing.border.*;


/**
 * Das AudioButtonPanel (abgeleitet von JPanel) erzeugt das fenster 
 * mit der audio-funktionalitt. Zudem sind auch die betreffenden
 * listeners hier zu finden.
 */


public class AudioButtonPanel extends JPanel
					{

	private JButton playButton, stopButton;
	private JTextField textField;
	private JSlider DSlider;

	private DftRoot dftRoot;

	private TitledBorder titleBorder= BorderFactory.createTitledBorder(
					BorderFactory.createEtchedBorder(), " A U D I O ");
	private TitledBorder volumeTitledBorder=BorderFactory.createTitledBorder(
					BorderFactory.createEtchedBorder()," Volume ");
	private TitledBorder statusTitledBorder= BorderFactory.createTitledBorder(
					BorderFactory.createEtchedBorder()," Driver Status ");

	public AudioButtonPanel(DftRoot dftRootA) {
     		this.dftRoot=dftRootA;
		

	        playButton = new JButton("Play");
        	stopButton = new JButton("Stop");

		DSlider = new JSlider(JSlider.HORIZONTAL,0, 100, 100);
		DSlider.setMajorTickSpacing(0);
		DSlider.setMinorTickSpacing(0);
		DSlider.setPaintTicks(true);
		DSlider.setPaintLabels(false);
		DSlider.setBorder(volumeTitledBorder);

		textField= new JTextField();
		textField.setEditable(false); 
		textField.setBorder(statusTitledBorder);
		textField.setText("Device Driver Working Properly");
        
	      	playButton.addActionListener(new ActionListener() {
        	    public void actionPerformed(ActionEvent e) {
            
			dftRoot.flushAudio(441,0,(dftRoot.ANZ_SAMPLES)/2);
	  		dftRoot.getAudioOutput().playSound();
            	}
        	});
        	stopButton.addActionListener(new ActionListener() {
        	    public void actionPerformed(ActionEvent e) {
        	    	dftRoot.getAudioOutput().stopSound();
        	    }
        	});
    
	    
		DSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
        			JSlider source = (JSlider)e.getSource();	
	    			//int fps = (int)source.getValue();
				dftRoot.getAudioOutput().setVolume((double) source.getValue() );
    			}
		});


		setLayout(null);

		add(playButton);
		add(stopButton);
		add(textField);
		add(DSlider);


		playButton.setBounds(10 + getInsets().left, 20 + getInsets().top, 60, 50);
	        stopButton.setBounds(10 + getInsets().left, 85 + getInsets().top, 60, 50);
      
		textField.setBounds(80 + getInsets().left, 20 + getInsets().top, 190, 50);
		DSlider.setBounds(80 + getInsets().left, 85 + getInsets().top, 190, 50);

		setBorder(titleBorder);
		installAudioErrorChannel();

    	}



	public void enableAudioButtons(){
		playButton.setEnabled(true);
		stopButton.setEnabled(true);
		DSlider.setEnabled(true);
		textField.setText("Device driver working properly");
	}

	public void disableAudioButtons(){
		playButton.setEnabled(false);
		stopButton.setEnabled(false);
		DSlider.setEnabled(false);
		textField.setText("Device driver not working");
	}

	private void installAudioErrorChannel(){
		dftRoot.getAudioOutput().setAudioErrorChannel(this);
		dftRoot.getAudioOutput().obtainAndOpenLine();
	}


}
