/* This file is part of VisualDFT.
 *
 * VisualDFT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * VisualDFT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VisualDFT; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package ch.educeth.visualdft;

import javax.sound.sampled.*;

/**
 * Die klasse AudioOutput ist zustndig fr die erzeugung von sound.
 * Sie versucht einen audio kanal zu ffnen. Gelingt dies,
 * so wird das "signal" mit hilfe des "standard audio device drivers" ausgegeben.
 */

class AudioOutput {


	byte soundByteArray[]=new byte[4410];
	private AudioFormat format=new AudioFormat(44100,16,1,true,false);
	private Clip clip=null;
	private AudioButtonPanel audioButtonPanel=null;
	private double volume=100.0d;

	public AudioOutput(){}


	public void obtainAndOpenLine(){
		enableAudio();

		DataLine.Info info = new DataLine.Info(Clip.class,format); 
		if (!AudioSystem.isLineSupported(info)) {
			disableAudio();
		}
		// Obtain and open the line.
		try {
		clip = (Clip) AudioSystem.getLine(info);
		} catch (LineUnavailableException ex) {	disableAudio();	}

		try{
		clip.open(format,soundByteArray,0,4408);
		} catch(Exception eee){ disableAudio(); }
	}

	public void setVolume(double volume){this.volume=volume;}


	public void setSample(double doubleSample,int position){
		int sample;		

		sample=(int) (doubleSample*270*volume);

		soundByteArray[0+2*position+0]=(byte) (sample%256);
		soundByteArray[1+2*position+0]=(byte) (sample/256);

		soundByteArray[0+2*position+882]=(byte)(sample%256);
		soundByteArray[1+2*position+882]=(byte)(sample/256);

		soundByteArray[0+2*position+1764]=(byte)(sample%256);
		soundByteArray[1+2*position+1764]=(byte)(sample/256);

		soundByteArray[0+2*position+2646]=(byte)(sample%256);
		soundByteArray[1+2*position+2646]=(byte)(sample/256);

		soundByteArray[0+2*position+3528]=(byte)(sample%256);
		soundByteArray[1+2*position+3528]=(byte)(sample/256);
	}

	public void playSound(){	
		try{
		clip.close();
		clip.open(format,soundByteArray,0,4408);
		} catch(Exception eee){disableAudio();}
		clip.loop(40);
	}

	public void stopSound(){
		clip.stop();
	}

	
	public void setAudioErrorChannel(AudioButtonPanel audioButtonPanel){
		this.audioButtonPanel=audioButtonPanel;
	}


	public void enableAudio(){audioButtonPanel.enableAudioButtons();}

	public void disableAudio(){audioButtonPanel.disableAudioButtons();}






}