/* This file is part of VisualDFT.
 *
 * VisualDFT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * VisualDFT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VisualDFT; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package ch.educeth.visualdft;

import java.lang.reflect.*;

/**
 * Hier befinden sich die methoden fr die eigentliche fourier transformation.
 * 
 * 1. DFT: Berechnet mit hilfe der methode SkalarPNorm (skalarprodukt+normierung) 
 *         die fourrier transformation "vorwrts".
 *         d.h. INPUT:  (SttzstellenVektorMatrix,Amplituden-Zeit-Array)
 *              OUTPUT: (Amplituden-Frequenz-Matrix)
 * 
 * 
 * 2. DFTInv: Berechnet die fourrier transformation "rckwrts". 
 *            d.h. INPUT:  (Amplituden-Frequenz-Matrix)
 *                 OUTPUT: (Amplituden-Zeit-Array)
 */



public class DftOps {

	public DftOps(){}


	/* (Inputs: StuetzstellenVektorMatrix,Amplituden-Zeit-Array)
	   (Output: Amplituden-Frequenz-Matrix) */
	
	public static void DFT(SVector[][] stVeMatrix,double[] amplTimeArray,
						      double[][] amplFreqArray){
		int I,Le;

		Le=(stVeMatrix[0][0].GetLength())/2+1;

		for (I=0; I< Le ; I++) {				
			amplFreqArray[0][I]= SkalarPNorm(amplTimeArray,stVeMatrix[0][I]);
		}
			
		for (I=1; I< Le-1 ; I++) {				
			amplFreqArray[1][I]= SkalarPNorm(amplTimeArray,stVeMatrix[1][I]);
		}
		amplFreqArray[1][0]=0;
		amplFreqArray[1][Le-1]=0;

	}

	public static void DFTInv(double d,double[] amplTimeArray,double[][] amplFreqArray){
		int I,Le=0;

		Le=Array.getLength(amplTimeArray);
		for (I=0; I< Le ; I++) {				
			amplTimeArray[I]=FunkAuswertung(I*d,amplFreqArray,0,Le/2);
		}
	}



	public static double SkalarPNorm(double[] amplTimeArray,SVector sVec){
		int i,le;
		double Res=0.0d,ZB=0.0d,Betrag=0.0d;		


		le=sVec.GetLength();
		for (i=0; i< le ; i++) {				
			ZB=sVec.Read(i);
			Res+= (amplTimeArray[i]*ZB);
			Betrag+= ZB*ZB;
		}
		return Res/Betrag;
	}

	public static double FunkAuswertung(double pos,double[][] amplFreqArray,int under,int upper){
		int i,le;
		double res=0.0d;

		le=java.lang.reflect.Array.getLength(amplFreqArray[0]);
		for (i=under; i< upper+1 ; i++) {				
			res+=amplFreqArray[0][i]*Math.cos(i*pos);
		}
		for (i=under; i< upper+1 ; i++) {				
			res+=amplFreqArray[1][i]*Math.sin(i*pos);
		}

		return res;
	}



}