/* This file is part of VisualDFT.
 *
 * VisualDFT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * VisualDFT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VisualDFT; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package ch.educeth.visualdft;

import java.io.*;
import java.math.*;
import java.awt.geom.*;

/**
 * Diese klasse ist der operative kernpunkt des applets. Alle GUI objekte besitzen  
 * eine referenz auf dieses object. Nicht alle dft-operationen sind hier zu finden,
 * sie werden jedoch stets von hier aus weiterdelegiert.
 * 
 * Alle datenstrukturen,die fr die durchfhrung der fourier transformation gebraucht
 * werden sind hier zu finden:
 * amplTimeArray: Alle x-koordinaten der punkte welche im "Amplituden/Zeit-Fenster" dargestellt sind
 * 
 * amplFreqArray: Alle koeffizienten (cos UND sin) die im "Frequenz/Zeit-Fenster" gezeichnet werden 
 * 
 * stVeMatrix: Die sttzstellenvektormatrix ist eine matrix von vektoren, die beim start des applets 
 *             einmalig berechnet wird und bei jeder spteren durchfhrung
 *             der fourier transformation verwendet wird.         
 */

public class DftRoot
			{

	private VisualDFT visualDFT;
	private AudioOutput audioOutput=new AudioOutput();

	static final int ANZ_SAMPLES=20;
	static final double DELTA= (2*Math.PI / ((double) ANZ_SAMPLES)) ;

	int Under=1,Upper=0;
	boolean Solo=false;

	private double[] amplTimeArray={-0.5d,-0.4d,-0.2d,0.7d,0.3d,-0.4d,0.0d,-0.4d,-0.1d,-0.2d,
					0.3d,-0.7d,0.4d,-0.1d,-0.3d,0.7d,0.3d,0.5d,-0.6d,-0.7d};
	private double[][] amplFreqArray=new double[2][ANZ_SAMPLES/2+1];
	private SVector[][] stVeMatrix;



    	public DftRoot(VisualDFT visualDFT) {
		this.visualDFT=visualDFT;
		createStVeMatrix();
		dftForward();
  	}


	private void createStVeMatrix(){
		int i,j;
		SVector Vect;

		stVeMatrix= new SVector[2][ANZ_SAMPLES/2+1];
		
		for (i=0; i< (ANZ_SAMPLES/2+1) ; i++) {
			
			Vect = new SVector(ANZ_SAMPLES);
			for (j=0; j<ANZ_SAMPLES; j++) {
				Vect.Write(j , Math.cos(i*j*DELTA) );
			}		
			stVeMatrix[0][i]=Vect;	
		}



		for (i=0; i< (ANZ_SAMPLES/2+1) ; i++) {
			
			Vect = new SVector(ANZ_SAMPLES);
			for (j=0; j<ANZ_SAMPLES; j++) {
				Vect.Write(j , Math.sin(i*j*DELTA) );
			}		
			stVeMatrix[1][i]=Vect;
		}
	}


	public void CalcGraph(int prec,GeneralPath graphShape,int under,int upper){
		int i;
		double rndDelta=(2*Math.PI-DELTA)/prec;
		
		graphShape.reset();
		
		/* Dont Draw in this Case */
		if (upper==-1){return;} 	

		graphShape.moveTo((float)0.0d , (float)DftOps.FunkAuswertung(0.0d,amplFreqArray,under,upper));
		
		for (i=1; i< prec+1 ; i++) {
	  		graphShape.lineTo((float)rndDelta*i,
				(float)DftOps.FunkAuswertung(rndDelta*i,amplFreqArray,under,upper));
		}
	}

	public void dftForward(){DftOps.DFT(stVeMatrix,amplTimeArray,amplFreqArray);}

	public void dftInverse(){DftOps.DFTInv(DELTA,amplTimeArray,amplFreqArray);}


	public void setAmplTimePoint(int pos,double value){
		amplTimeArray[pos]=value;
	}

	public double getAmplTimePoint(int pos){
		return amplTimeArray[pos];
	}


	public void setAmplFreqTupple(int pos,double valueCos,double valueSin){
		amplFreqArray[0][pos]=valueCos;
		amplFreqArray[1][pos]=valueSin;
	}

	public double getAmplFreq(int cosorsin, int pos){
		return amplFreqArray[cosorsin][pos];	
	}


	public AudioOutput getAudioOutput(){return audioOutput;}	

	public void flushAudio(int prec,int under,int upper){
		int i;
		
		double audioDelta=(2*Math.PI)/prec;
		
		for (i=0; i< prec ; i++) {
		audioOutput.setSample( (double)DftOps.FunkAuswertung(audioDelta*i,amplFreqArray,under,upper),i);
		}

	}

	public void redrawall(){
		visualDFT.restorep();
	}

}